<?php

/**
 * @file
 * Definition of WoW\RemoteEntity.
 */

/**
 * Defines a remote entity class.
 */
abstract class WoWRemoteEntity extends Entity {

  /**
   * Timestamp for entity's last fetch.
   *
   * @var integer
   */
  public $lastFetched = 0;

  /**
   * Timestamp for entity's last update.
   *
   * @var integer
   */
  public $lastModified = 0;

  /**
   * Fetch entity from the service and refresh itself.
   *
   * @param array $query
   *   (Optional) An array of query key/value-pairs (without any URL-encoding) to
   *   append to the URL.
   *   - locale: You can specify your own locale here.
   *     It it the responsibility of the caller to pass a valid locale.
   *     @see wow_api_locale()
   * @param array $options
   *   (Optional) An array that can have one or more of the following elements:
   *   - headers: An array containing request headers to send as name/value pairs.
   *   - method: A string containing the request method. Defaults to 'GET'.
   *   - data: A string containing the request body, formatted as
   *     'param=value&param=value&...'. Defaults to NULL.
   *   - max_redirects: An integer representing how many times a redirect
   *     may be followed. Defaults to 3.
   *   - timeout: A float representing the maximum number of seconds the function
   *     call may take. The default is 30 seconds. If a timeout occurs, the error
   *     code is set to the HTTP_REQUEST_TIMEOUT constant.
   *   - context: A context resource created with stream_context_create().
   *
   * @return WoWRemoteEntity $entity
   *   Return itself.
   */
  public function fetch(array $query = array(), array $options = array()) {
    return entity_get_controller($this->entityType)->fetch($this, $query, $options);
  }

  public function merge(array $values) {
    foreach ($values as $key => $value) {
      $this->{$key} = $value;
    }
  }

  /**
   * Return the entity remote URI.
   */
  abstract public function remoteURI();
}