<?php

/**
 * @file
 * Definition of CharacterController.
 */

/**
 * Controller class for characters.
 *
 * This extends the EntityAPIController class, adding required special handling
 * for character objects.
 */
class WoWCharacterController extends WoWRemoteEntityController {

  /**
   * @param WoWCharacter $character
   * @param DatabaseTransaction $transactin
   */
  public function save($character, DatabaseTransaction $transaction = NULL) {
    $transaction = isset($transaction) ? $transaction : db_transaction();
    // Load the stored entity, if any.
    if (!empty($character->{$this->idKey}) && !isset($character->original)) {
      // In order to properly work in case of name changes, load the original
      // entity using the id key if it is available.
      $character->original = entity_load_unchanged($this->entityType, $character->{$this->idKey});
    }

    // We are about to set main status of a character.
    if ((!empty($character->is_new) && $character->isMain)
      || !empty($character->original) && $character->isMain != $character->original->isMain) {

      // Invokes set_main hook on behalf the wow_character entity. Modules
      // implementers can check the entity and set the isMain property to FALSE.
      $this->invoke('set_main', $character);

      // If the character is about to be main, update other to ensure unicity.
      if ($character->isMain) {
        db_update('wow_characters')
          ->fields(array('isMain' => 0))
          ->condition('uid', $character->uid)
          ->condition('cid', $character->cid, '<>')
          ->execute();
      }
    }

    return parent::save($character, $transaction);
  }

  /**
   * @param WoWCharacter $character
   * @param WoWHttpResult $result
   */
  protected function processResult($character, $result) {
    $character->realm = wow_realm_to_slug($result->getArray('realm'));
    $character->lastModified = $result->getArray('lastModified') / 1000;
  }

  /**
   * @param WoWCharacter $character
   * @param WoWHttpStatus $status
   */
  protected function processStatus($character, $status) {
    switch ($status->getCode()) {
      // The status returned is 304 Not Modified.
      case 304:
        // The character was not modified since the lastModified timestamp.
        $character->save();
        break;

      // The status returned is 404 Not Found.
      case 404:
        // The character was existing but can't be found anymore. Deletes the
        // local character.
        $character->delete();

      // The status code could not be handled here.
      default:
        throw new WoWHttpException($status);
        break;
    }
  }
}
